@@ -48,6 +48,8 @@ class ProxyListenerDynamoDB(ProxyListener):
         if method == 'OPTIONS':
             return 200
 
+        if not data:
+            data = '{}'
         data = json.loads(to_str(data))
         ddb_client = aws_stack.connect_to_service('dynamodb')
         action = headers.get('X-Amz-Target')
@@ -275,12 +277,13 @@ class ProxyListenerDynamoDB(ProxyListener):
             return
 
         elif action == '%s.DeleteTable' % ACTION_PREFIX:
+            table_arn = json.loads(response._content).get('TableDescription', {}).get('TableArn')
             event_publisher.fire_event(
                 event_publisher.EVENT_DYNAMODB_DELETE_TABLE,
                 payload={'n': event_publisher.get_hash(data['TableName'])}
             )
-
-            TABLE_TAGS.pop(json.loads(response._content).get('TableDescription', {}).get('TableArn', None), None)
+            self.delete_all_event_source_mappings(table_arn)
+            TABLE_TAGS.pop(table_arn, None)
 
             return
 
@@ -399,6 +402,14 @@ class ProxyListenerDynamoDB(ProxyListener):
                 records.append(new_record)
         return records
 
+    def delete_all_event_source_mappings(self, table_arn):
+        if table_arn:
+            lambda_client = aws_stack.connect_to_service('lambda')
+            result = lambda_client.list_event_source_mappings(EventSourceArn=table_arn)
+            for event in result['EventSourceMappings']:
+                event_source_mapping_id = event['UUID']
+                lambda_client.delete_event_source_mapping(UUID=event_source_mapping_id)
+
     @staticmethod
     def _thread_local(name, default=None):
         try:
@@ -429,6 +440,8 @@ def find_existing_item(put_item, table_name=None):
 
     req = {'TableName': table_name, 'Key': search_key}
     existing_item = aws_stack.dynamodb_get_item_raw(req)
+    if not existing_item:
+        return existing_item
     if 'Item' not in existing_item:
         if 'message' in existing_item:
             table_names = ddb_client.list_tables()['TableNames']
@@ -487,7 +500,11 @@ def forward_to_lambda(records):
             'Records': [record]
         }
         for src in sources:
-            lambda_api.run_lambda(event=event, context={}, func_arn=src['FunctionArn'])
+            if src.get('State') != 'Enabled':
+                continue
+
+            lambda_api.run_lambda(event=event, context={}, func_arn=src['FunctionArn'],
+                asynchronous=not config.SYNCHRONOUS_DYNAMODB_EVENTS)
 
 
 def forward_to_ddb_stream(records):
